<?php

namespace Litecms\Instructor\Http\Controllers;

use Exception;
use Litepie\Http\Controllers\ResourceController as BaseController;
use Litepie\Repository\Filter\RequestFilter;
use Litecms\Instructor\Forms\Instructor as InstructorForm;
use Litecms\Instructor\Http\Requests\InstructorRequest;
use Litecms\Instructor\Interfaces\InstructorRepositoryInterface;
use Litecms\Instructor\Repositories\Eloquent\Filters\InstructorResourceFilter;
use Litecms\Instructor\Repositories\Eloquent\Presenters\InstructorListPresenter;

/**
 * Resource controller class for instructor.
 */
class InstructorResourceController extends BaseController
{

    /**
     * Initialize instructor resource controller.
     *
     *
     * @return null
     */
    public function __construct(InstructorRepositoryInterface $instructor)
    {
        parent::__construct();
        $this->form = InstructorForm::setAttributes()->toArray();
        $this->modules = $this->modules(config('litecms.instructor.modules'), 'instructor', guard_url('instructor'));
        $this->repository = $instructor;
    }

    /**
     * Display a list of instructor.
     *
     * @return Response
     */
    public function index(InstructorRequest $request)
    {

        $pageLimit = $request->input('pageLimit', config('database.pagination.limit'));
        $data = $this->repository
            ->pushFilter(RequestFilter::class)
            ->pushFilter(InstructorResourceFilter::class)
            ->setPresenter(InstructorListPresenter::class)
            ->simplePaginate($pageLimit)
            // ->withQueryString()
            ->toArray();

        extract($data);
        $form = $this->form;
        $modules = $this->modules;

        return $this->response->setMetaTitle(trans('instructor::instructor.names'))
            ->view('instructor::instructor.index')
            ->data(compact('data', 'meta', 'links', 'modules', 'form'))
            ->output();
    }

    /**
     * Display instructor.
     *
     * @param Request $request
     * @param Model   $instructor
     *
     * @return Response
     */
    public function show(InstructorRequest $request, InstructorRepositoryInterface $repository)
    {
        $form = $this->form;
        $modules = $this->modules;
        $data = $repository->toArray();
        return $this->response
            ->setMetaTitle(trans('app.view') . ' ' . trans('instructor::instructor.name'))
            ->data(compact('data', 'form', 'modules', 'form'))
            ->view('instructor::instructor.show')
            ->output();
    }

    /**
     * Show the form for creating a new instructor.
     *
     * @param Request $request
     *x
     * @return Response
     */
    public function create(InstructorRequest $request, InstructorRepositoryInterface $repository)
    {
        $form = $this->form;
        $modules = $this->modules;
        $data = $repository->toArray();
        return $this->response->setMetaTitle(trans('app.new') . ' ' . trans('instructor::instructor.name'))
            ->view('instructor::instructor.create')
            ->data(compact('data', 'form', 'modules'))
            ->output();
    }

    /**
     * Create new instructor.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(InstructorRequest $request, InstructorRepositoryInterface $repository)
    {
        try {
            $attributes = $request->all();
            $attributes['user_id'] = user_id();
            $attributes['user_type'] = user_type();
            $repository->create($attributes);
            $data = $repository->toArray();

            return $this->response->message(trans('messages.success.created', ['Module' => trans('instructor::instructor.name')]))
                ->code(204)
                ->data(compact('data'))
                ->status('success')
                ->url(guard_url('instructor/instructor/' . $data['id']))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('/instructor/instructor'))
                ->redirect();
        }

    }

    /**
     * Show instructor for editing.
     *
     * @param Request $request
     * @param Model   $instructor
     *
     * @return Response
     */
    public function edit(InstructorRequest $request, InstructorRepositoryInterface $repository)
    {
        $form = $this->form;
        $modules = $this->modules;
        $data = $repository->toArray();

        return $this->response->setMetaTitle(trans('app.edit') . ' ' . trans('instructor::instructor.name'))
            ->view('instructor::instructor.edit')
            ->data(compact('data', 'form', 'modules'))
            ->output();
    }

    /**
     * Update the instructor.
     *
     * @param Request $request
     * @param Model   $instructor
     *
     * @return Response
     */
    public function update(InstructorRequest $request, InstructorRepositoryInterface $repository)
    {
        try {
            $attributes = $request->all();
            $repository->update($attributes);
            $data = $repository->toArray();

            return $this->response->message(trans('messages.success.updated', ['Module' => trans('instructor::instructor.name')]))
                ->code(204)
                ->status('success')
                ->data(compact('data'))
                ->url(guard_url('instructor/instructor/' . $data['id']))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('instructor/instructor/' .  $repository->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove the instructor.
     *
     * @param Model   $instructor
     *
     * @return Response
     */
    public function destroy(InstructorRequest $request, InstructorRepositoryInterface $repository)
    {
        try {
            $repository->delete();
            $data = $repository->toArray();

            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('instructor::instructor.name')]))
                ->code(202)
                ->status('success')
                ->data(compact('data'))
                ->url(guard_url('instructor/instructor/0'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('instructor/instructor/' .  $repository->getRouteKey()))
                ->redirect();
        }

    }
}
